import 'package:flutter/material.dart';
import 'package:flutter_neumorphic/flutter_neumorphic.dart';

import 'package:formio_flutter/formio_flutter.dart';
import 'package:formio_flutter/src/abstraction/abstraction.dart';
import 'package:formio_flutter/src/constants/constants.dart';
import 'package:formio_flutter/src/models/models.dart';
import 'package:formio_flutter/src/providers/providers.dart';

import '../utils/utils.dart';

/// Extends the abstract class [WidgetParser]
class NumberTextFieldParser extends WidgetParser {
  /// Returns a [Widget] of type [NumberTextField]
  @override
  Widget parse(
    Component map,
    BuildContext context,
    ClickListener listener,
    WidgetProvider widgetProvider,
  ) {
    return NumberTextFieldCreator(
      map: map,
      widgetProvider: widgetProvider,
    );
  }

  /// [widgetName] => "number"
  @override
  String get widgetName => 'number';
}

// ignore: must_be_immutable
class NumberTextFieldCreator extends StatefulWidget implements Manager {
  NumberTextFieldCreator({this.map, this.widgetProvider});

  final Component map;
  final WidgetProvider widgetProvider;
  final controller = TextEditingController();

  @override
  _NumberTextFieldCreatorState createState() => _NumberTextFieldCreatorState();

  /// Returns a [String] with the value contained inside [Component.key]
  @override
  String get keyValue => map.key ?? 'numberField';

  /// Current value of the [Widget]
  @override
  String get data => controller.text ?? '';
}

class _NumberTextFieldCreatorState extends State<NumberTextFieldCreator> {
  String characters = '';
  final Map<String, dynamic> _mapper = {};

  @override
  void initState() {
    super.initState();

    _mapper[widget.map.key] = {''};

    if (widget.map.defaultValue != null) {
      if (widget.map.defaultValue is List<String>) {
        widget.controller.text = widget.map.defaultValue
            .asMap()
            .values
            .toString()
            .replaceAll(RegExp('[()]'), '');
      } else if (widget.map.defaultValue != '') {
        widget.controller.text = double.parse(
          widget.map.defaultValue.toString(),
        ).toStringAsFixed(widget.map.decimalLimit);
      } else {
        widget.controller.text = '*';
      }
    }

    widget.controller.addListener(() {
      _mapper.update(widget.map.key, (value) => widget.controller.value.text);
    });

    WidgetsBinding.instance.addPostFrameCallback((_) {
      _mapper.update(widget.map.key, (value) {
        // if it's a calculatedValue, add the operation
        if (widget.map.calculateValue != null) {
          return '''${AppConstants.drops}${AppConstants.multiply}${AppConstants.concentration}''';
        }
        // if it's not, just assign textEditingController's value
        return widget.controller.value.text;
      }, ifAbsent: () {
        _mapper.putIfAbsent(widget.map.key, () {
          if (widget.map.calculateValue != null) {
            return '''${AppConstants.drops}${AppConstants.multiply}${AppConstants.concentration}''';
          }
        });
      });
      widget.widgetProvider.widgetBloc.registerMap(_mapper);
    });
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    var isVisible = true;
    final size = MediaQuery.of(context).size;

    return StreamBuilder(
      stream: widget.widgetProvider.widgetBloc.widgetsStream,
      builder: (context, AsyncSnapshot<Map<String, dynamic>> snapshot) {
        isVisible = (widget.map.conditional != null && snapshot.data != null)
            ? (snapshot.data.containsKey(widget.map.conditional.when) &&
                    snapshot.data[widget.map.conditional.when].toString() ==
                        widget.map.conditional.eq)
                ? widget.map.conditional.show
                : !widget.map.conditional.show
            : true;

        if (!isVisible) {
          widget.controller.text = '';
          return const SizedBox.shrink();
        } else if (widget.map.calculateValue == null) {
          return Neumorphic(
            child: Container(
              width: (size.width * 0.5),
              padding: EdgeInsets.symmetric(horizontal: 4),
              child: TextField(
                enabled: !widget.map.disabled,
                obscureText: widget.map.mask,
                keyboardType: parsetInputType(widget.map.type),
                style: TextStyle(fontSize: 17, color: Colors.black),
                controller: widget.controller,
                onChanged: (value) {
                  if (value != '') {
                    value = double.parse(value).toStringAsFixed(
                      widget.map.decimalLimit,
                    );

                    _mapper.update(widget.map.key, (nVal) => value);
                    widget.widgetProvider.widgetBloc.registerMap(_mapper);
                  }

                  final drops = AppConstants.drops.toLowerCase();

                  if (widget.map.key.toLowerCase().contains(drops)) {
                    final concentrationKey = widget.map.key.replaceAll(
                      AppConstants.drops,
                      AppConstants.concentration,
                    );

                    final concentration = '${snapshot.data[concentrationKey]}';

                    final expressions = concentration.split(
                      AppConstants.multiply,
                    );

                    final newValue = concentration.replaceAll(
                      expressions.last,
                      value,
                    );

                    snapshot.data[concentrationKey] = newValue;
                    widget.widgetProvider.widgetBloc.registerMap(_mapper);
                  }

                  setState(() => characters = value);
                },
                decoration: InputDecoration(
                  counter: widget.map.showWordCount != null &&
                          widget.map.showWordCount
                      ? characters != ''
                          ? Text(
                              '''${characters.replaceAll(' ', '').length} numbers''')
                          : const SizedBox.shrink()
                      : null,
                  prefixText: widget.map.prefix ?? '',
                  prefixStyle: TextStyle(
                    color: Colors.black,
                    fontSize: 20,
                    fontWeight: FontWeight.w600,
                    background: Paint()..color = Colors.teal.shade200,
                  ),
                  labelText: widget.map.label ?? '',
                  hintText: widget.map.label,
                  suffixText: widget.map.suffix ?? '',
                  suffixStyle: TextStyle(),
                ),
              ),
            ),
          );
        } else if (snapshot.data != null) {
          return Neumorphic(
            padding: EdgeInsets.symmetric(horizontal: 5),
            child: ListTile(
              leading: Text(widget.map.label),
              title: Text(
                parseStringMultiply(snapshot.data[widget.map.key].toString()),
                style: TextStyle(fontSize: 17, color: Colors.black),
              ),
            ),
          );
        } else {
          return const SizedBox.shrink();
        }
      },
    );
  }
}
