import 'package:flutter/material.dart';
import 'package:flutter_neumorphic/flutter_neumorphic.dart';
import 'package:formio_flutter/src/constants/constants.dart';
import 'package:http/http.dart' as http;

import 'package:formio_flutter/formio_flutter.dart';
import 'package:formio_flutter/src/abstraction/abstraction.dart';
import 'package:formio_flutter/src/models/models.dart';
import 'package:formio_flutter/src/providers/providers.dart';

/// Extends the abstract class [WidgetParser]
class SelectParser extends WidgetParser {
  /// Returns a [Widget] of type [DropDown/Selector]
  @override
  Widget parse(
    Component map,
    BuildContext context,
    ClickListener listener,
    WidgetProvider widgetProvider,
  ) {
    return SelectParserWidget(map: map, widgetProvider: widgetProvider);
  }

  /// [widgetName] => "select"
  @override
  String get widgetName => 'select';
}

// ignore: must_be_immutable
class SelectParserWidget extends StatefulWidget implements Manager {
  Value selected;
  final Component map;
  final WidgetProvider widgetProvider;

  SelectParserWidget({this.map, this.widgetProvider});

  @override
  _SelectParserWidgetState createState() => _SelectParserWidgetState();

  /// Returns a [String] with the value contained inside [Component.key]
  @override
  String get keyValue => map.key ?? 'selectField';

  /// Current value of the [Widget]
  @override
  Object get data => selected != null ? selected.toMap() : '';
}

class _SelectParserWidgetState extends State<SelectParserWidget> {
  bool isVisible = true;
  Future<List<Value>> _futureValues;
  List<DropdownMenuItem<Value>> _values;
  final Map<String, dynamic> _mapper = {};

  /// When the [url] isn't null or empty then the data is prefetched for the [Select] widget.
  Future<List<Value>> _makeRequest() async {
    final client = http.Client();
    final uri = Uri.parse(widget.map.data.url);
    final response = await client.get(uri);
    final result = Value.valuesFromJson(response.body);

    if (result.isNotEmpty) {
      setupDropDown(result);
      _mapper[widget.map.key] = result.first.value;
      widget.widgetProvider.widgetBloc.registerMap(_mapper);
    }
    return result;
  }

  /// Setup all the functionality for the dropDown widget.
  void setupDropDown(List<Value> values) async {
    _values = buildDropDownItems(values);

    if (widget.map.defaultValue != null && widget.map.defaultValue != '') {
      var _position = 0;

      if (widget.map.defaultValue is Map<String, dynamic>) {
        _position = values.indexWhere(
          (element) => element.value == widget.map.defaultValue['value'],
        );
      } else if (widget.map.defaultValue is String) {
        _position = values.indexWhere(
          (element) => element.value == widget.map.defaultValue,
        );
      }

      if (_position == -1) {
        widget.selected = _values.first.value;
      } else {
        widget.selected = _values[_position].value;
      }
    } else {
      widget.selected = _values.first.value;
    }
  }

  @override
  void initState() {
    super.initState();
    if (widget.map.data.url.isEmpty &&
        widget.map.data.values.isNotEmpty &&
        widget.map.data.values.first.value != null) {
      setupDropDown(widget.map.data.values);

      WidgetsBinding.instance.addPostFrameCallback((_) {
        _mapper[widget.map.key] = _values.first.value.value;
        widget.widgetProvider.widgetBloc.registerMap(_mapper);
      });
    }

    if (widget.map.data.url.isNotEmpty) _futureValues ??= _makeRequest();
  }

  @override
  void dispose() {
    super.dispose();
  }

  List<DropdownMenuItem<Value>> buildDropDownItems(List listItems) {
    final items = <DropdownMenuItem<Value>>[];
    if (listItems == null ||
        listItems.first == null ||
        (listItems.first as Value).value == null) {
      items.add(DropdownMenuItem(
        value: null,
        child: Text('Empty'),
      ));
    } else {
      for (Value listItem in listItems) {
        items.add(
          DropdownMenuItem(
            value: listItem,
            child: Text(listItem.label),
          ),
        );
      }
    }
    return items;
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.symmetric(vertical: 4.0),
      child: StreamBuilder(
        stream: widget.widgetProvider.widgetBloc.widgetsStream,
        builder: (context, AsyncSnapshot<Map<String, dynamic>> snapshot) {
          if (widget.map.conditional != null) {
            isVisible = (widget.map.conditional != null &&
                    snapshot.data != null)
                ? (snapshot.data.containsKey(widget.map.conditional.when) &&
                        snapshot.data[widget.map.conditional.when].toString() ==
                            widget.map.conditional.eq)
                    ? widget.map.conditional.show
                    : !widget.map.conditional.show
                : true;
          }

          if (!isVisible) return const SizedBox.shrink();

          return Neumorphic(
            padding: EdgeInsets.symmetric(horizontal: 4.0),
            child: Flex(
              direction: Axis.vertical,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                NeumorphicText(
                  widget.map.label == null || widget.map.label.isEmpty
                      ? ''
                      : widget.map.label,
                  textStyle: NeumorphicTextStyle(
                    fontSize: 17,
                    fontWeight: FontWeight.w500,
                  ),
                  style: NeumorphicStyle(
                    depth: 13,
                    intensity: 0.9,
                    color: Colors.black,
                  ),
                ),
                if (widget.map.data.url.isNotEmpty)
                  FutureBuilder(
                    future: _futureValues,
                    builder: (_, AsyncSnapshot<List<Value>> valueSnapshot) {
                      switch (valueSnapshot.connectionState) {
                        case ConnectionState.done:
                          return DropdownButton<Value>(
                            hint: NeumorphicText(
                              widget.map.label,
                              style: NeumorphicStyle(
                                depth: 13,
                                intensity: 0.9,
                                color: Colors.black,
                              ),
                            ),
                            isExpanded: true,
                            style: TextStyle(
                              fontSize: 14,
                              fontWeight: FontWeight.w400,
                              color: Colors.black,
                            ),
                            value: widget.selected,
                            items: _values,
                            onChanged: (value) {
                              if (!widget.map.disabled) {
                                _mapper.update(
                                  widget.map.key,
                                  (_) => value.value,
                                );

                                widget.widgetProvider.widgetBloc.registerMap(
                                  _mapper,
                                );
                              }

                              if (value.data.concentrationPerDrop != null) {
                                final concentrationKey =
                                    widget.map.key.replaceAll(
                                  AppConstants.used,
                                  AppConstants.concentration,
                                );

                                final concentration =
                                    snapshot.data[concentrationKey];

                                final expressions = concentration.split(
                                  AppConstants.multiply,
                                );

                                final cpd = value.data.concentrationPerDrop;

                                // _mapper.update(
                                //   concentrationKey,
                                //   (nVal) {
                                //     return '$cpd${AppConstants.multiply}${expressions.last}';
                                //   },
                                //   ifAbsent: () {
                                //     _mapper.addAll({
                                //       concentrationKey:
                                //           '$cpd${AppConstants.multiply}${expressions.last}',
                                //     });
                                //   },
                                // );

                                snapshot.data[concentrationKey] =
                                    '$cpd${AppConstants.multiply}${expressions.last}';

                                // widget.widgetProvider.widgetBloc.registerMap(
                                //   _mapper,
                                // );
                              }

                              setState(() => widget.selected = value);
                            },
                          );
                          break;
                        default:
                          return CircularProgressIndicator();
                          break;
                      }
                    },
                  )
                else if (widget.map.data.values.isNotEmpty &&
                    widget.map.data.values.first.value != null)
                  DropdownButton<Value>(
                    hint: NeumorphicText(
                      widget.map.label,
                      style: NeumorphicStyle(
                        depth: 13.0,
                        intensity: 0.90,
                        color: Colors.black,
                      ),
                    ),
                    isExpanded: true,
                    style: TextStyle(
                      fontSize: 14.0,
                      fontWeight: FontWeight.w400,
                      color: Colors.black,
                    ),
                    value: widget.selected,
                    items: _values,
                    onChanged: (value) {
                      if (!widget.map.disabled) {
                        _mapper.update(widget.map.key, (nVal) => value.value);
                        widget.widgetProvider.widgetBloc.registerMap(_mapper);
                        setState(() => widget.selected = value);
                      }
                    },
                  )
                else
                  DropdownButton<Value>(
                    hint: NeumorphicText(
                      widget.map.label,
                      style: NeumorphicStyle(
                        depth: 13,
                        intensity: 0.9,
                        color: Colors.black,
                      ),
                    ),
                    isExpanded: true,
                    style: TextStyle(
                      color: Colors.black,
                      fontSize: 14,
                      fontWeight: FontWeight.w400,
                    ),
                    value: widget.selected,
                    items: _values,
                    onChanged: (value) {
                      if (!widget.map.disabled) {
                        _mapper.update(widget.map.key, (nVal) => value.value);
                        widget.widgetProvider.widgetBloc.registerMap(_mapper);
                        setState(() => widget.selected = value);
                      }
                    },
                  ),
              ],
            ),
          );
        },
      ),
    );
  }
}
